#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Optimization;
using System;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  /// <summary>
  /// Abstract base class symbolic data analysis solutions
  /// </summary>
  [StorableClass]
  public abstract class SymbolicDataAnalysisSolution : ResultCollection, ISymbolicDataAnalysisSolution {
    private const string ModelResultName = "Model";
    private const string ProblemDataResultName = "ProblemData";
    #region properties
    public ISymbolicDataAnalysisModel Model {
      get { return (ISymbolicDataAnalysisModel)this[ModelResultName].Value; }
      set {
        if (this[ModelResultName].Value != value) {
          if (value != null) {
            this[ModelResultName].Value = value;
            OnModelChanged(EventArgs.Empty);
          }
        }
      }
    }

    IDataAnalysisModel IDataAnalysisSolution.Model {
      get { return Model; }
    }

    public IDataAnalysisProblemData ProblemData {
      get { return (IDataAnalysisProblemData)this[ProblemDataResultName].Value; }
      set {
        if (this[ProblemDataResultName].Value != value) {
          if (value != null) {
            this[ProblemDataResultName].Value = value;
            OnProblemDataChanged(EventArgs.Empty);
          }
        }
      }
    }
    #endregion

    public event EventHandler ModelChanged;

    public event EventHandler ProblemDataChanged;

    [StorableConstructor]
    protected SymbolicDataAnalysisSolution(bool deserializing) : base(deserializing) { }
    protected SymbolicDataAnalysisSolution(SymbolicDataAnalysisSolution original, Cloner cloner)
      : base(original, cloner) {
    }
    public SymbolicDataAnalysisSolution()
      : base() {
      Add(new Result(ModelResultName, "The symbolic data analysis model.", typeof(IDataAnalysisModel)));
      Add(new Result(ProblemDataResultName, "The symbolic data analysis problem data.", typeof(IDataAnalysisProblemData)));
    }

    protected virtual void OnModelChanged(EventArgs e) {
      var listeners = ModelChanged;
      if (listeners != null) listeners(this, e);
    }

    protected virtual void OnProblemDataChanged(EventArgs e) {
      var listeners = ProblemDataChanged;
      if (listeners != null) listeners(this, e);
    }
  }
}
